// robowaifu access control
// ========================
// -The software testbed for devising access control systems for robowaifus

// -Filename: Info.hpp
// -The declarations interface of the Info class

// TODO DOC: Add exposition for Anon as a learner, to understand why & how
// Info objects are used in this system.

#pragma once

#include <string>

#include "Human.hpp"
#include "utility.hpp"

using std::string;

/** Represents a piece of information within robowaifu's model-of-the-world,
 * with the associated Human ownership & authorized container of Relation's
 * (Permit) embedded within */
class Info {
 public:
  Info(){};
  /** Basic Info ctor
   * @param data [IN] this Info's data load
   * @param owner [IN] the Human in control of this Info
   * @param permits [IN] the container of Relation's this Info is authorized for
   */
  Info(std::string const& data, Human const& owner, Permit const& permits);

  /** @return the Info's data load */
  auto info() const noexcept -> decltype(std::string{});

  /** @return the Info's container of Relation's it is authorized for */
  auto allows() const noexcept -> decltype(Permit{});

 private:
  string data;
  Human  owner;
  Permit permits;
};

// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
